import asyncio

from channels.testing import WebsocketCommunicator
from django.test import TransactionTestCase
from unittest.mock import patch, AsyncMock
from channels.routing import URLRouter

from whitebox.routing import websocket_urlpatterns
from plugin.manager import plugin_manager


async def deactivate_websocket_loop_after_delay(plugin, delay):
    await asyncio.sleep(delay)
    plugin.is_active = False


class TestWhiteboxPluginStratuxIntegration(TransactionTestCase):
    def setUp(self):
        plugin_manager.plugins = []
        plugin_manager.previously_discovered_plugins = {}
        plugin_manager.discover_plugins()

        self.plugin = next(
            (
                x
                for x in plugin_manager.plugins
                if x.__class__.__name__ == "WhiteboxPluginStratux"
            ),
            None,
        )
        self.application = URLRouter(websocket_urlpatterns)

        return super().setUp()

    def test_plugin_loaded(self):
        self.assertIsNotNone(self.plugin)

    async def test_websocket_flight_start_initializes_tasks(self):
        communicator = WebsocketCommunicator(self.application, "/ws/flight/")
        connected, _ = await communicator.connect()
        self.assertTrue(connected)

        with (
            patch.object(self.plugin, "gather_traffic", AsyncMock()),
            patch.object(self.plugin, "gather_situation", AsyncMock()),
            patch.object(self.plugin, "gather_status", AsyncMock()),
        ):
            await communicator.send_json_to({"type": "flight_start"})
            response = await communicator.receive_json_from()

            self.assertEqual(response["type"], "message")
            self.assertEqual(response["message"], "Flight started")

            self.assertTrue(self.plugin.is_active)

            self.assertIsNotNone(self.plugin.traffic_task)
            self.assertIsNotNone(self.plugin.situation_task)
            self.assertIsNotNone(self.plugin.status_task)

            self.plugin.gather_traffic.assert_called_once()
            self.plugin.gather_situation.assert_called_once()
            self.plugin.gather_status.assert_called_once()

        await communicator.disconnect()
